package app.views.match.components;

import app.data.entity.Game;
import app.data.entity.Match;
import app.data.service.ChessComService;
import app.data.service.MatchService;
import app.gameimage.GameImageService;
import app.navigation.Navigation;
import app.utils.ChessComUtils;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.ComponentEventListener;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.shared.Registration;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class EditMatchCard extends VerticalLayout {

    private final Navigation navigation;
    private final MatchService matchService;
    private final ChessComService chessComService;
    private final GameImageService gameImageService;

    private final ArrayList<TextField> textFields = new ArrayList<>();
    // TODO: autocorrect "/live/game" to "/game/live/"
    private final Button submitButton = new Button("Submit", new Icon(VaadinIcon.CHECK));
    // TODO: disable when no 6 values in form, and make sure you don't freeze forever when there are wrong entries
    // TODO: use overlapping progress here
    private Registration editSubmitButtonRegistration;
    private final Button editCancelButton = new Button("Cancel", new Icon(VaadinIcon.CLOSE));
    private Registration editCancelButtonRegistration;
    private final Button chessComButton = new Button("Autofill with the latest 6 games between the players", new Icon(VaadinIcon.MAGIC));
    private Registration chessComButtonButtonRegistration;

    private Match match;

    public EditMatchCard(Navigation navigation, ChessComService chessComService, GameImageService gameImageService) {
        this.navigation = navigation;
        this.chessComService = chessComService;
        this.matchService = navigation.getMatchService();
        this.gameImageService = gameImageService;

        defineLayout();
    }

    ////////////
    // LAYOUT //
    ////////////

    private void defineLayout() {
        setAlignItems(FlexComponent.Alignment.CENTER);
        setWidth("");
        addClassName("wrapper");
        add(chessComButton);

        defineTextFields();
        defineSubmitButton();
        addSubmitAndCancelButtons();
    }

    private void defineTextFields() {
        for (int i = 0; i < 6; i++) {
            TextField textField = new TextField();
            textField.setWidth("23em");
            HorizontalLayout horizontalLayout = new HorizontalLayout(new Label(String.format("Game %d:", i + 1)), textField);
            horizontalLayout.setAlignItems(FlexComponent.Alignment.CENTER);
            horizontalLayout.setJustifyContentMode(FlexComponent.JustifyContentMode.CENTER);
            add(horizontalLayout);
            textFields.add(textField);
        }
    }

    private void defineSubmitButton() {
        submitButton.addThemeVariants(ButtonVariant.LUMO_PRIMARY);
    }

    private void addSubmitAndCancelButtons() {
        HorizontalLayout buttonLayout = new HorizontalLayout();
        buttonLayout.setAlignItems(FlexComponent.Alignment.CENTER);
        buttonLayout.setJustifyContentMode(FlexComponent.JustifyContentMode.CENTER);
        buttonLayout.add(submitButton, editCancelButton);
        add(buttonLayout);
    }

    /////////////
    // CONTENT //
    /////////////

    void configureContent(Match match) {
        this.match = match;
        List<Game> games = new ArrayList<>(match.getGames());

        clearTextFields();
        fillTextFieldsWithURLs(games);

        configureChessComButton();
        configureEditSubmitButton();
        configureEditCancelButton();
    }

    private void clearTextFields() {
        textFields.forEach(textField -> textField.setValue(""));
    }

    private void fillTextFieldsWithURLs(List<Game> games) {
        for (int i = 0; i < Math.min(games.size(), 6); i++)
            textFields.get(i).setValue(ChessComUtils.getGameURL(games.get(i)));
    }

    private void configureChessComButton() {
        if (chessComButtonButtonRegistration != null) chessComButtonButtonRegistration.remove();
        chessComButtonButtonRegistration = chessComButton.addClickListener(createChessComButtonClickListener());
    }

    private ComponentEventListener<ClickEvent<Button>> createChessComButtonClickListener() {
        return event -> {
            List<Game> gamesBetweenPlayers = chessComService.getLatestGamesBetweenPlayers(match, 6, 2);
            fillTextFieldsWithURLs(gamesBetweenPlayers);
        };
    }

    private void configureEditSubmitButton() {
        if (editSubmitButtonRegistration != null) editSubmitButtonRegistration.remove();
        editSubmitButtonRegistration = submitButton.addClickListener(createEditSubmitButtonClickListener());
    }

    private ComponentEventListener<ClickEvent<Button>> createEditSubmitButtonClickListener() {
        return event -> {
            match.getGames().clear();
            addGamesToMatchFromFieldValues();
            matchService.update(match);
            match.getGames().forEach(gameImageService::createImageIfNotPresent);

            navigation.setEditFlag(false);
        };
    }

    private void addGamesToMatchFromFieldValues() {
        for (TextField textField : textFields) {
            Optional<Game> game = chessComService.getGame(textField.getValue(), match); // TODO: handle this when Optional is empty!
            game.ifPresent(value -> match.getGames().add(value));
        }
    }

    private void configureEditCancelButton() {
        if (editCancelButtonRegistration != null) editCancelButtonRegistration.remove();
        editCancelButtonRegistration = editCancelButton.addClickListener(createEditCancelButtonListener());
    }

    private ComponentEventListener<ClickEvent<Button>> createEditCancelButtonListener() {
        return event -> UI.getCurrent().navigate(String.format("matchday/%s", navigation.getWildcardParam().replace("edit/", "")));
    }
}
